module Wire

push!(LOAD_PATH, pwd())
using WWUMeshes
using GfxBase
using LinearAlgebra

using Luxor

function cube_to_tris()
    cube = cube_mesh()
    triangles = []
    for tri in cube.triangles
        vertices = [] 
        for v = 1:3
            push!(vertices, Vec4(cube.positions[tri.positions[v]]..., 1.0))
        end
        push!(triangles, vertices)
    end
    triangles
end

function main()

    # Viewport specs
    cw = ch = 400 # canvas dimensions

    # Ex 1. Construct a viewport matrix that maps the canonical
    #       view volume onto a ch-by-cw canvas.
    vp   = [1 0 0 0;
            0 1 0 0;
            0 0 1 0;
            0 0 0 1]

   # Orthographic projection matrix specs
    # view volume boundaries:
    l = -2  # left
    r = 2   # right
    b = -2  # bottom
    t = 2   # top
    n = 2   # near
    f = -10 # far (z)

    # Ex 2. Construct an orthographic projection matrix that maps
    #       the given viewport to normalized device coordinates. 
    # TODO: replace this matrix
    proj = [1 0 0 0;
            0 1 0 0;
            0 0 1 0;
            0 0 0 1]

    # Camera matrix specs
    eye = Vec3(-2, 1, 2)
    at = Vec3(0, 0, 0)
    up = Vec3(0, 1, 0)

    # done for you: construct a frame given the above
    view = at - eye
    w = -normalize(view)
    u = normalize(cross(up, w))
    v = normalize(cross(w, u))
    p = eye

    # Ex 3. Construct a camera matrix given the above basis. 
    # Possibly useful functions: hcat, vcat, inv
    # TODO
    cam = [1 0 0 0;
           0 1 0 0;
           0 0 1 0;
           0 0 0 1]

    # Model matrix
    model =[1 0 0 0;
            0 1 0 0;
            0 0 1 0;
            0 0 0 1]

    M = vp*proj*cam*model

    Drawing(cw, ch)
    background("white")
    setline(0.5)

    for tri in cube_to_tris()
        # transform each vertex
        v1 = M * tri[1]
        v2 = M * tri[2]
        v3 = M * tri[3]
        v1 = Point(v1[1], v1[2]) # drop z and w!
        v2 = Point(v2[1], v2[2])
        v3 = Point(v3[1], v3[2])
        line(v1, v2, :stroke)
        line(v2, v3, :stroke)
        line(v3, v1, :stroke)

    end
    finish()
    preview()
end


end # module Wire

Wire.main()